gnunet-svn
[Top][All Lists]
Advanced

[Date Prev][Date Next][Thread Prev][Thread Next][Date Index][Thread Index]

[taler-merchant] 29/277: implement POST /products


From: gnunet
Subject: [taler-merchant] 29/277: implement POST /products
Date: Sun, 05 Jul 2020 20:49:02 +0200

This is an automated email from the git hooks/post-receive script.

grothoff pushed a commit to branch master
in repository merchant.

commit fc99580fd44c983c9bdb67be210a67e5d9466376
Author: Christian Grothoff <christian@grothoff.org>
AuthorDate: Tue Apr 21 19:22:59 2020 +0200

    implement POST /products
---
 src/include/taler_merchant_service.h |   4 +-
 src/lib/Makefile.am                  |   1 +
 src/lib/merchant_api_post_products.c | 306 +++++++++++++++++++++++++++++++++++
 3 files changed, 309 insertions(+), 2 deletions(-)

diff --git a/src/lib/Makefile.am b/src/lib/Makefile.am
index a762b00..efcea87 100644
--- a/src/lib/Makefile.am
+++ b/src/lib/Makefile.am
@@ -29,6 +29,7 @@ libtalermerchant_la_SOURCES = \
   merchant_api_patch_instance.c \
   merchant_api_patch_product.c \
   merchant_api_post_instances.c \
+  merchant_api_post_products.c \
   merchant_api_check_payment.c \
   merchant_api_history.c \
   merchant_api_proposal.c \
diff --git a/src/lib/merchant_api_post_products.c 
b/src/lib/merchant_api_post_products.c
new file mode 100644
index 0000000..b15866d
--- /dev/null
+++ b/src/lib/merchant_api_post_products.c
@@ -0,0 +1,306 @@
+/*
+  This file is part of TALER
+  Copyright (C) 2020 Taler Systems SA
+
+  TALER is free software; you can redistribute it and/or modify
+  it under the terms of the GNU Lesser General Public License as
+  published by the Free Software Foundation; either version 2.1,
+  or (at your option) any later version.
+
+  TALER is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU Lesser General Public License for more details.
+
+  You should have received a copy of the GNU Lesser General
+  Public License along with TALER; see the file COPYING.LGPL.
+  If not, see <http://www.gnu.org/licenses/>
+*/
+/**
+ * @file lib/merchant_api_post_products.c
+ * @brief Implementation of the POST /products request
+ *        of the merchant's HTTP API
+ * @author Christian Grothoff
+ */
+#include "platform.h"
+#include <curl/curl.h>
+#include <jansson.h>
+#include <microhttpd.h> /* just for HTTP status codes */
+#include <gnunet/gnunet_util_lib.h>
+#include "taler_merchant_service.h"
+#include <taler/taler_json_lib.h>
+#include <taler/taler_curl_lib.h>
+
+
+/**
+ * Handle for a POST /products/$ID operation.
+ */
+struct TALER_MERCHANT_ProductsPostHandle
+{
+
+  /**
+   * The url for this request.
+   */
+  char *url;
+
+  /**
+   * Handle for the request.
+   */
+  struct GNUNET_CURL_Job *job;
+
+  /**
+   * Function to call with the result.
+   */
+  TALER_MERCHANT_ProductsPostCallback cb;
+
+  /**
+   * Closure for @a cb.
+   */
+  void *cb_cls;
+
+  /**
+   * Reference to the execution context.
+   */
+  struct GNUNET_CURL_Context *ctx;
+
+  /**
+   * Minor context that holds body and headers.
+   */
+  struct TALER_CURL_PostContext post_ctx;
+
+};
+
+
+/**
+ * Function called when we're done processing the
+ * HTTP POST /products request.
+ *
+ * @param cls the `struct TALER_MERCHANT_ProductsPostHandle`
+ * @param response_code HTTP response code, 0 on error
+ * @param json response body, NULL if not in JSON
+ */
+static void
+handle_post_products_finished (void *cls,
+                               long response_code,
+                               const void *response)
+{
+  struct TALER_MERCHANT_ProductsPostHandle *pph = cls;
+  const json_t *json = response;
+  struct TALER_MERCHANT_HttpResponse hr = {
+    .http_status = (unsigned int) response_code,
+    .reply = json
+  };
+
+  pph->job = NULL;
+  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+              "POST /products completed with response code %u\n",
+              (unsigned int) response_code);
+  switch (response_code)
+  {
+  case 0:
+    hr.ec = TALER_EC_INVALID_RESPONSE;
+    break;
+  case MHD_HTTP_NO_CONTENT:
+    break;
+  case MHD_HTTP_BAD_REQUEST:
+    hr.ec = TALER_JSON_get_error_code (json);
+    hr.hint = TALER_JSON_get_error_hint (json);
+    /* This should never happen, either us
+     * or the merchant is buggy (or API version conflict);
+     * just pass JSON reply to the application */
+    break;
+  case MHD_HTTP_FORBIDDEN:
+    hr.ec = TALER_JSON_get_error_code (json);
+    hr.hint = TALER_JSON_get_error_hint (json);
+    /* Nothing really to verify, merchant says we tried to abort the payment
+     * after it was successful. We should pass the JSON reply to the
+     * application */
+    break;
+  case MHD_HTTP_NOT_FOUND:
+    hr.ec = TALER_JSON_get_error_code (json);
+    hr.hint = TALER_JSON_get_error_hint (json);
+    /* Nothing really to verify, this should never
+       happen, we should pass the JSON reply to the
+       application */
+    break;
+  case MHD_HTTP_CONFLICT:
+    hr.ec = TALER_JSON_get_error_code (json);
+    hr.hint = TALER_JSON_get_error_hint (json);
+    break;
+  case MHD_HTTP_INTERNAL_SERVER_ERROR:
+    hr.ec = TALER_JSON_get_error_code (json);
+    hr.hint = TALER_JSON_get_error_hint (json);
+    /* Server had an internal issue; we should retry,
+       but this API leaves this to the application */
+    break;
+  default:
+    TALER_MERCHANT_parse_error_details_ (json,
+                                         response_code,
+                                         &hr);
+    /* unexpected response code */
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                "Unexpected response code %u/%d\n",
+                (unsigned int) response_code,
+                (int) hr.ec);
+    GNUNET_break_op (0);
+    break;
+  }
+  pph->cb (pph->cb_cls,
+           &hr);
+  TALER_MERCHANT_products_post_cancel (pph);
+}
+
+
+/**
+ * Make a POST /products request to add a product to the
+ * inventory.
+ *
+ * @param ctx the context
+ * @param backend_url HTTP base URL for the backend
+ * @param instance_id instance to add a product to,
+ *                    NULL to query the default instance
+ * @param product_id identifier to use for the product
+ * @param description description of the product
+ * @param description_i18n Map from IETF BCP 47 language tags to localized 
descriptions
+ * @param unit unit in which the product is measured (liters, kilograms, 
packages, etc.)
+ * @param price the price for one @a unit of the product, zero is used to 
imply that
+ *              this product is not sold separately or that the price is not 
fixed and
+ *              must be supplied by the front-end.  If non-zero, price must 
include
+ *              applicable taxes.
+ * @param image base64-encoded product image
+ * @param taxes list of taxes paid by the merchant
+ * @param total_stocked in @a units, -1 to indicate "infinite" (i.e. 
electronic books)
+ * @param address where the product is in stock
+ * @param next_restock when the next restocking is expected to happen, 0 for 
unknown,
+ *                     #GNUNET_TIME_UNIT_FOREVER_ABS for 'never'.
+ * @param cb function to call with the backend's result
+ * @param cb_cls closure for @a cb
+ * @return the request handle; NULL upon error
+ */
+struct TALER_MERCHANT_ProductsPostHandle *
+TALER_MERCHANT_products_post (
+  struct GNUNET_CURL_Context *ctx,
+  const char *backend_url,
+  const char *instance_id,
+  const char *product_id,
+  const char *description,
+  const json_t *description_i18n,
+  const char *unit,
+  const struct TALER_Amount *price,
+  const json_t *image,
+  const json_t *taxes,
+  int64_t total_stocked,
+  const json_t *address,
+  struct GNUNET_TIME_Absolute next_restock,
+  TALER_MERCHANT_ProductsPostCallback cb,
+  void *cb_cls)
+{
+  struct TALER_MERCHANT_ProductsPostHandle *pph;
+  json_t *req_obj;
+
+  req_obj = json_pack ("{s:s, s:s, s:O, s:s, s:o,"
+                       " s:O, s:O, s:I, s:O, s:o}",
+                       "product_id",
+                       product_id,
+                       "description",
+                       description,
+                       "description_i18n",
+                       description_i18n,
+                       "unit",
+                       unit,
+                       "price",
+                       TALER_JSON_from_amount (price),
+                       /* End of first group of 5 */
+                       "image",
+                       image,
+                       "taxes",
+                       taxes,
+                       "total_stocked",
+                       (json_int_t) total_stocked,
+                       "address",
+                       address,
+                       "next_restock",
+                       GNUNET_JSON_from_time_abs (next_restock));
+  if (NULL == req_obj)
+  {
+    GNUNET_break (0);
+    return NULL;
+  }
+  pph = GNUNET_new (struct TALER_MERCHANT_ProductsPostHandle);
+  pph->ctx = ctx;
+  pph->cb = cb;
+  pph->cb_cls = cb_cls;
+  {
+    char *path;
+
+    if (NULL == instance_id)
+      GNUNET_asprintf (&path,
+                       "products");
+    else
+      GNUNET_asprintf (&path,
+                       "instances/%s/products",
+                       instance_id);
+    pph->url = TALER_url_join (backend_url,
+                               path,
+                               NULL);
+    GNUNET_free (path);
+  }
+  if (NULL == pph->url)
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                "Could not construct request URL.\n");
+    json_decref (req_obj);
+    GNUNET_free (pph);
+    return NULL;
+  }
+  {
+    CURL *eh;
+
+    eh = curl_easy_init ();
+    if (GNUNET_OK !=
+        TALER_curl_easy_post (&pph->post_ctx,
+                              eh,
+                              req_obj))
+    {
+      GNUNET_break (0);
+      json_decref (req_obj);
+      GNUNET_free (pph);
+      return NULL;
+    }
+
+    json_decref (req_obj);
+    GNUNET_assert (CURLE_OK == curl_easy_setopt (eh,
+                                                 CURLOPT_URL,
+                                                 pph->url));
+    pph->job = GNUNET_CURL_job_add2 (ctx,
+                                     eh,
+                                     pph->post_ctx.headers,
+                                     &handle_post_products_finished,
+                                     pph);
+  }
+  return pph;
+}
+
+
+/**
+ * Cancel /products request.  Must not be called by clients after
+ * the callback was invoked.
+ *
+ * @param igh request to cancel.
+ */
+void
+TALER_MERCHANT_products_post_cancel (
+  struct TALER_MERCHANT_ProductsPostHandle *pph)
+{
+  if (NULL != pph->job)
+  {
+    GNUNET_CURL_job_cancel (pph->job);
+    pph->job = NULL;
+  }
+  TALER_curl_easy_post_finished (&pph->post_ctx);
+  GNUNET_free (pph->url);
+  GNUNET_free (pph);
+}
+
+
+/* end of merchant_api_post_products.c */

diff --git a/src/include/taler_merchant_service.h 
b/src/include/taler_merchant_service.h
index ba7723c..54a53c1 100644
--- a/src/include/taler_merchant_service.h
+++ b/src/include/taler_merchant_service.h
@@ -877,7 +877,7 @@ typedef void
  * @param image base64-encoded product image
  * @param taxes list of taxes paid by the merchant
  * @param total_stocked in @a units, -1 to indicate "infinite" (i.e. 
electronic books)
  * @param location where the product is in stock
  * @param next_restock when the next restocking is expected to happen, 0 for 
unknown,
  *                     #GNUNET_TIME_UNIT_FOREVER_ABS for 'never'.
  * @param cb function to call with the backend's result
@@ -897,7 +897,7 @@ TALER_MERCHANT_products_post (
   const json_t *image,
   const json_t *taxes,
   int64_t total_stocked,
-  const json_t *location,
+  const json_t *address,
   struct GNUNET_TIME_Absolute next_restock,
   TALER_MERCHANT_ProductsPostCallback cb,
   void *cb_cls);
@@ -877,7 +877,7 @@ typedef void
  * @param total_stocked in @a units, -1 to indicate "infinite" (i.e. 
electronic books)
- * @param location where the product is in stock
+ * @param address where the product is in stock
  * @param next_restock when the next restocking is expected to happen, 0 for 
unknown,
-- 
To stop receiving notification emails like this one, please contact
gnunet@gnunet.org.



reply via email to

[Prev in Thread] Current Thread [Next in Thread]